@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Specialized callbacks that track similarity metrics during training"""
-from typing import Dict, List, Sequence, Union
+from typing import Dict, List, Optional, Sequence, Union
 from pathlib import Path
 import math
 
@@ -21,10 +21,11 @@ import tensorflow as tf
 from tensorflow.keras.callbacks import Callback
 
 from .classification_metrics import ClassificationMetric
-from .classification_metrics import make_classification_metric # noqa
+from .classification_metrics import make_classification_metric  # noqa
+from .matchers import ClassificationMatch
 from .evaluators import Evaluator, MemoryEvaluator
 from .models import SimilarityModel
-from .types import Tensor, IntTensor
+from .types import Tensor, FloatTensor, IntTensor
 from .utils import unpack_lookup_distances, unpack_lookup_labels
 
 
@@ -37,15 +38,20 @@ class EvalCallback(Callback):
 
     """
 
-    def __init__(self,
-                 queries: Tensor,
-                 query_labels: Sequence[int],
-                 targets: Tensor,
-                 target_labels: Sequence[int],
-                 distance: str = 'cosine',
-                 metrics: Sequence[Union[str, ClassificationMetric]] = ['binary_accuracy', 'f1score'],  # noqa
-                 tb_logdir: str = None,
-                 k: int = 1):
+    def __init__(
+            self,
+            queries: Tensor,
+            query_labels: Sequence[int],
+            targets: Tensor,
+            target_labels: Sequence[int],
+            distance: str = 'cosine',
+            metrics: Sequence[Union[str, ClassificationMetric]] = [
+                'binary_accuracy', 'f1score'
+            ],  # noqa
+            tb_logdir: str = None,
+            k: int = 1,
+            matcher: Union[str, ClassificationMatch] = 'match_nearest',
+            distance_thresholds: Optional[FloatTensor] = None):
         """Evaluate model matching quality against a validation dataset at
         epoch end.
 
@@ -68,22 +74,37 @@ class EvalCallback(Callback):
 
             tb_logdir: Where to write TensorBoard logs. Defaults to None.
 
-            k: How many neighbors to retrieve for evaluation. Defaults to 1.
+            k: The number of nearest neighbors to return for each query.
+
+            matcher: {'match_nearest', 'match_majority_vote'} or
+            ClassificationMatch object. Defines the classification matching,
+            e.g., match_nearest will count a True Positive if the query_label is
+            equal to the label of the nearest neighbor and the distance is less
+            than or equal to the distance threshold.
+
+            distance_thresholds: A 1D tensor denoting the distances points at
+            which we compute the metrics. If None, distance_thresholds is set to
+            tf.constant([math.inf])
         """
         super().__init__()
         self.queries = queries
-        self.query_labels: IntTensor = tf.cast(
-                tf.convert_to_tensor(query_labels),
-                dtype='int32'
-        )
+        if not tf.is_tensor(query_labels):
+            query_labels = tf.convert_to_tensor(np.array(query_labels))
+        self.query_labels: IntTensor = tf.cast(query_labels, dtype='int32')
         self.targets = targets
         self.target_labels = target_labels
         self.distance = distance
-        self.k = k
         self.evaluator = MemoryEvaluator()
         # typing requires this weird formulation of creating a new list
-        self.metrics: List[ClassificationMetric] = (
-                [make_classification_metric(m) for m in metrics])
+        self.metrics: List[ClassificationMetric] = ([
+            make_classification_metric(m) for m in metrics
+        ])
+        self.k = k
+        self.matcher = matcher
+        if distance_thresholds is not None:
+            self.distance_thresholds = distance_thresholds
+        else:
+            self.distance_thresholds = tf.constant([math.inf])
 
         if tb_logdir:
             tb_logdir = str(Path(tb_logdir) / 'index/')
@@ -93,6 +114,12 @@ class EvalCallback(Callback):
             self.tb_writer = None
 
     def on_epoch_end(self, epoch: int, logs: dict = None):
+        """Computes the eval metrics at the end of each epoch.
+
+        NOTE: This method resets the index and batch adds the target embeddings
+        to the index using the new embeddings generated by the current version
+        of the model.
+        """
         if logs is None:
             logs = {}
 
@@ -102,14 +129,15 @@ class EvalCallback(Callback):
         # rebuild the index
         self.model.index(self.targets, self.target_labels, verbose=0)
 
-        results = _evaluate_classification(
-                queries=self.queries,
-                query_labels=self.query_labels,
-                model=self.model,
-                evaluator=self.evaluator,
-                k=self.k,
-                metrics=self.metrics
-        )
+        results = _compute_classification_metrics(
+            queries=self.queries,
+            query_labels=self.query_labels,
+            model=self.model,
+            evaluator=self.evaluator,
+            metrics=self.metrics,
+            k=self.k,
+            matcher=self.matcher,
+            distance_thresholds=self.distance_thresholds)
 
         mstr = []
         for metric_name, vals in results.items():
@@ -120,6 +148,10 @@ class EvalCallback(Callback):
                 with self.tb_writer.as_default():
                     tf.summary.scalar(metric_name, float_val, step=epoch)
 
+        # reset the index to prevent users from accidently using this after the
+        # callback
+        self.model.reset_index()
+
         print(' - '.join(mstr))
 
 
@@ -137,16 +169,22 @@ class SplitValidationLoss(Callback):
     unseen classes and may provide insight into how well the embedding will
     generalize to new classes.
     """
-    def __init__(self,
-                 queries: Tensor,
-                 query_labels: Sequence[int],
-                 targets: Tensor,
-                 target_labels: Sequence[int],
-                 known_classes: IntTensor,
-                 distance: str = 'cosine',
-                 metrics: Sequence[Union[str, ClassificationMetric]] = ['binary_accuracy', 'f1score'],  # noqa
-                 tb_logdir: str = None,
-                 k=1):
+
+    def __init__(
+            self,
+            queries: Tensor,
+            query_labels: Sequence[int],
+            targets: Tensor,
+            target_labels: Sequence[int],
+            known_classes: IntTensor,
+            distance: str = 'cosine',
+            metrics: Sequence[Union[str, ClassificationMetric]] = [
+                'binary_accuracy', 'f1score'
+            ],  # noqa
+            tb_logdir: str = None,
+            k: int = 1,
+            matcher: Union[str, ClassificationMatch] = 'match_nearest',
+            distance_thresholds: Optional[FloatTensor] = None):
         """Creates the validation callbacks.
 
         Args:
@@ -170,17 +208,35 @@ class SplitValidationLoss(Callback):
 
             tb_logdir: Where to write TensorBoard logs. Defaults to None.
 
-            k: How many neighbors to retrieve for evaluation. Defaults to 1.
+            k: The number of nearest neighbors to return for each query. The
+            lookups are consumed by the Matching Strategy and used to derive the
+            matching label and distance.
+
+            matcher: {'match_nearest', 'match_majority_vote'} or
+            ClassificationMatch object. Defines the classification matching,
+            e.g., match_nearest will count a True Positive if the query_label
+            is equal to the label of the nearest neighbor and the distance is
+            less than or equal to the distance threshold.
+
+            distance_thresholds: A 1D tensor denoting the distances points at
+            which we compute the metrics. If None, distance_thresholds is set to
+            tf.constant([math.inf])
         """
         super().__init__()
         self.targets = targets
         self.target_labels = target_labels
         self.distance = distance
-        self.k = k
         self.evaluator = MemoryEvaluator()
         # typing requires this weird formulation of creating a new list
-        self.metrics: List[ClassificationMetric] = (
-                [make_classification_metric(m) for m in metrics])
+        self.metrics: List[ClassificationMetric] = ([
+            make_classification_metric(m) for m in metrics
+        ])
+        self.k = k
+        self.matcher = matcher
+        if distance_thresholds is not None:
+            self.distance_thresholds = distance_thresholds
+        else:
+            self.distance_thresholds = tf.constant([math.inf])
 
         if tb_logdir:
             tb_logdir = str(Path(tb_logdir) / 'index/')
@@ -189,7 +245,8 @@ class SplitValidationLoss(Callback):
         else:
             self.tb_writer = None
 
-        query_labels = tf.convert_to_tensor(query_labels)
+        if not tf.is_tensor(query_labels):
+            query_labels = tf.convert_to_tensor(np.array(query_labels))
         query_labels = tf.cast(query_labels, dtype='int32')
 
         # Create separate validation sets for the known and unknown classes
@@ -204,33 +261,39 @@ class SplitValidationLoss(Callback):
         # See https://numpy.org/doc/stable/user/basics.broadcasting.html
         broadcast_classes = tf.expand_dims(known_classes, axis=0)
         broadcast_labels = tf.expand_dims(query_labels, axis=-1)
-        known_mask = tf.math.reduce_any(
-                broadcast_classes == broadcast_labels, axis=1)
+        known_mask = tf.math.reduce_any(broadcast_classes == broadcast_labels,
+                                        axis=1)
         known_idxs = tf.squeeze(tf.where(known_mask))
         unknown_idxs = tf.squeeze(tf.where(~known_mask))
 
         with tf.device("/cpu:0"):
             self.queries_known = tf.gather(queries, indices=known_idxs)
-            self.query_labels_known = (
-                    tf.gather(query_labels, indices=known_idxs))
+            self.query_labels_known = (tf.gather(query_labels,
+                                                 indices=known_idxs))
             # Expand to 2D if we only have a single example
             if tf.rank(self.queries_known) == 1:
-                self.queries_known = (
-                        tf.expand_dims(self.queries_known, axis=0))
-                self.query_labels_known = (
-                        tf.expand_dims(self.query_labels_known, axis=0))
+                self.queries_known = (tf.expand_dims(self.queries_known,
+                                                     axis=0))
+                self.query_labels_known = (tf.expand_dims(
+                    self.query_labels_known, axis=0))
 
             self.queries_unknown = tf.gather(queries, indices=unknown_idxs)
-            self.query_labels_unknown = (
-                    tf.gather(query_labels, indices=unknown_idxs))
+            self.query_labels_unknown = (tf.gather(query_labels,
+                                                   indices=unknown_idxs))
             # Expand to 2D if we only have a single example
             if tf.rank(self.queries_unknown) == 1:
-                self.queries_unknown = (
-                        tf.expand_dims(self.queries_unknown, axis=0))
-                self.query_labels_unknown = (
-                        tf.expand_dims(self.query_labels_unknown, axis=0))
+                self.queries_unknown = (tf.expand_dims(self.queries_unknown,
+                                                       axis=0))
+                self.query_labels_unknown = (tf.expand_dims(
+                    self.query_labels_unknown, axis=0))
 
     def on_epoch_end(self, epoch: int, logs: dict = None):
+        """Computes the eval metrics at the end of each epoch.
+
+        NOTE: This method resets the index and batch adds the target embeddings
+        to the index using the new embeddings generated by the current version
+        of the model.
+        """
         _ = epoch
         if logs is None:
             logs = {}
@@ -241,23 +304,25 @@ class SplitValidationLoss(Callback):
         # rebuild the index
         self.model.index(self.targets, self.target_labels, verbose=0)
 
-        known_results = _evaluate_classification(
-                queries=self.queries_known,
-                query_labels=self.query_labels_known,
-                model=self.model,
-                evaluator=self.evaluator,
-                k=self.k,
-                metrics=self.metrics
-        )
+        known_results = _compute_classification_metrics(
+            queries=self.queries_known,
+            query_labels=self.query_labels_known,
+            model=self.model,
+            evaluator=self.evaluator,
+            metrics=self.metrics,
+            k=self.k,
+            matcher=self.matcher,
+            distance_thresholds=self.distance_thresholds)
 
-        unknown_results = _evaluate_classification(
-                queries=self.queries_unknown,
-                query_labels=self.query_labels_unknown,
-                model=self.model,
-                evaluator=self.evaluator,
-                k=self.k,
-                metrics=self.metrics
-        )
+        unknown_results = _compute_classification_metrics(
+            queries=self.queries_unknown,
+            query_labels=self.query_labels_unknown,
+            model=self.model,
+            evaluator=self.evaluator,
+            metrics=self.metrics,
+            k=self.k,
+            matcher=self.matcher,
+            distance_thresholds=self.distance_thresholds)
 
         mstr = []
         for metric_name, vals in known_results.items():
@@ -278,17 +343,46 @@ class SplitValidationLoss(Callback):
                 with self.tb_writer.as_default():
                     tf.summary.scalar(full_metric_name, float_val, step=epoch)
 
+        # reset the index to prevent users from accidently using this after the
+        # callback
+        self.model.reset_index()
+
         print(' - '.join(mstr))
 
 
-def _evaluate_classification(
-                 queries: Tensor,
-                 query_labels: IntTensor,
-                 model: SimilarityModel,
-                 evaluator: Evaluator,
-                 k: int,
-                 metrics: Sequence[ClassificationMetric]
-                 ) -> Dict[str, np.ndarray]:
+def _compute_classification_metrics(
+        queries: Tensor, query_labels: IntTensor, model: SimilarityModel,
+        evaluator: Evaluator, metrics: Sequence[ClassificationMetric], k: int,
+        matcher: Union[str, ClassificationMatch],
+        distance_thresholds: FloatTensor) -> Dict[str, np.ndarray]:
+    """Compute the classification metrics.
+
+    Args:
+        queries: A Tensor of embeddings representing the queries.
+
+        query_labels: An IntTensor representing the class ids associated with
+            the queries.
+
+        model: The current similarity model.
+
+        evaluator: An Evalutar object for evaluating the index performance.
+
+        metrics: A list of classification metrics objects.
+
+        k: The number of nearest neighbors to return for each query.
+
+        matcher: {'match_nearest', 'match_majority_vote'} or ClassificationMatch
+        object. Defines the classification matching, e.g., match_nearest will
+        count a True Positive if the query_label is equal to the label of the
+        nearest neighbor and the distance is less than or equal to the distance
+        threshold.
+
+        distance_thresholds: A 1D tensor denoting the distances points at which
+        we compute the metrics.
+
+    Returns:
+        A Python dict mapping the metric name to the copmuted value.
+    """
     lookups = model.lookup(queries, k=k, verbose=0)
     lookup_distances = unpack_lookup_distances(lookups)
     lookup_labels = unpack_lookup_labels(lookups)
@@ -299,11 +393,11 @@ def _evaluate_classification(
         query_labels=query_labels,
         lookup_labels=lookup_labels,
         lookup_distances=lookup_distances,
-        distance_thresholds=tf.constant([math.inf]),
+        distance_thresholds=distance_thresholds,
         metrics=metrics,
-        matcher='match_nearest',
+        matcher=matcher,
         verbose=0)
-    
+
     # The callbacks don't set a distance theshold so we remove it here.
     results.pop('distance')
 
